"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getExceptionCode = getExceptionCode;
exports.handleErrorMessage = handleErrorMessage;
var _lodash = _interopRequireDefault(require("lodash.get"));
var _enums = require("../enums");
var _exceptions = require("../exceptions");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function handleErrorMessage(error) {
  const response = (0, _lodash.default)(error, 'response', null);
  if (response) {
    const status = (0, _lodash.default)(response, 'status', -1);
    const message = (0, _lodash.default)(response, 'message', '');
    if (status === _enums.HttpStatusCode.UNAUTHORIZED) {
      var _response$headers, _response$headers$www;
      if (response !== null && response !== void 0 && (_response$headers = response.headers) !== null && _response$headers !== void 0 && (_response$headers$www = _response$headers['www-authenticate']) !== null && _response$headers$www !== void 0 && _response$headers$www.includes('The token expired')) {
        return {
          code: _enums.HttpStatusCode.UNAUTHORIZED,
          message,
          status: _exceptions.BusinessExceptionCode.EXP_TOKEN
        };
      } else {
        return {
          code: _enums.HttpStatusCode.UNAUTHORIZED,
          message,
          status: _exceptions.BusinessExceptionCode.UNAUTHORIZED
        };
      }
    }
    if (message) {
      return {
        code: status,
        message: message,
        status: getExceptionCode(status)
      };
    }
    return {
      code: status,
      message: 'Error occurred. Please try again.',
      status: getExceptionCode(status)
    };
  } else {
    if (error && typeof error === 'object') {
      const status = (0, _lodash.default)(error, 'status', -1);
      const message = (0, _lodash.default)(error, 'message', '');
      const code = (0, _lodash.default)(error, 'code', _enums.HttpStatusCode.BAD_REQUEST);
      if (status === _enums.HttpStatusCode.UNAUTHORIZED) {
        var _response$headers2, _response$headers2$ww;
        if (response !== null && response !== void 0 && (_response$headers2 = response.headers) !== null && _response$headers2 !== void 0 && (_response$headers2$ww = _response$headers2['www-authenticate']) !== null && _response$headers2$ww !== void 0 && _response$headers2$ww.includes('The token expired')) {
          return {
            code: status,
            message,
            status: _exceptions.BusinessExceptionCode.EXP_TOKEN
          };
        } else {
          return {
            code: status,
            message,
            status: _exceptions.BusinessExceptionCode.UNAUTHORIZED
          };
        }
      }
      if (message) {
        return {
          code: code,
          message: message,
          status: getExceptionCode(status)
        };
      }
      return {
        code: code,
        message: 'Error occurred. Please try again.',
        status: getExceptionCode(status)
      };
    }
    return {
      code: _enums.HttpStatusCode.BAD_REQUEST,
      message: 'Error occurred. Please try again.',
      status: getExceptionCode(_enums.HttpStatusCode.BAD_REQUEST)
    };
  }
}
function getExceptionCode(statusCode) {
  switch (statusCode) {
    case _enums.HttpStatusCode.UNAUTHORIZED:
      return _exceptions.BusinessExceptionCode.UNAUTHORIZED;
    case _enums.HttpStatusCode.REQUEST_TIMEOUT:
      return _exceptions.BusinessExceptionCode.REQUEST_TIME_OUT;
    default:
      return _exceptions.BusinessExceptionCode.UNEXPECTED_ERROR;
  }
}
//# sourceMappingURL=handle-error-message.js.map