"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.invoke = invoke;
var _effects = require("redux-saga/effects");
var _exceptions = require("../exceptions");
var _repositories = require("../repositories");
var _reactNative = require("react-native");
var _enums = require("../enums");
var _handleErrorMessage = require("./handle-error-message");
function* invoke(options) {
  const {
    execution,
    handleError,
    showDialog,
    preAction,
    isShowMessageError = true
  } = options;
  try {
    const connect = yield (0, _effects.select)(state => state.connect);
    const {
      isConnected = false
    } = connect || {};
    yield (0, _effects.put)(_repositories.loadingRepo.actions.onFetching(preAction.type));

    /**
     * check connect to internet
     * */
    if (!isConnected) {
      _reactNative.DeviceEventEmitter.emit(_enums.EventKeys.LOST_CONNECT.toString());
      return;
    }
    if (showDialog) {
      yield (0, _effects.put)(_repositories.loadingRepo.actions.showLoading(preAction.type));
    }
    yield* execution();
    yield (0, _effects.put)(_repositories.loadingRepo.actions.nonFetching(preAction.type));
    if (showDialog) {
      yield (0, _effects.put)(_repositories.loadingRepo.actions.hideLoading(preAction.type));
    }
  } catch (error) {
    if (__DEV__) {
      console.info(`Saga Invoke Error [${preAction.type}]>>>>>`, {
        error
      });
    }
    yield (0, _effects.put)(_repositories.loadingRepo.actions.nonFetching(preAction.type));
    if (showDialog) {
      yield (0, _effects.put)(_repositories.loadingRepo.actions.hideLoading(preAction.type));
    }
    const errorMessage = global.___handleErrorMessage ? global.___handleErrorMessage(error) : (0, _handleErrorMessage.handleErrorMessage)(error);
    if (errorMessage && errorMessage.status === _exceptions.BusinessExceptionCode.UNEXPECTED_ERROR) {
      _reactNative.DeviceEventEmitter.emit(_enums.EventKeys.UNEXPECTED_ERROR, {
        error: errorMessage
      });
      return;
    }
    if (errorMessage && errorMessage.status === _exceptions.BusinessExceptionCode.EXP_TOKEN) {
      _reactNative.DeviceEventEmitter.emit(_enums.EventKeys.EXP_TOKEN.toString(), {
        error: errorMessage
      });
      return;
    }

    // if refresh token is expired or not valid -> logout
    if (errorMessage.code === 401 && errorMessage.status === _exceptions.BusinessExceptionCode.UNAUTHORIZED) {
      _reactNative.DeviceEventEmitter.emit(_enums.EventKeys.LOGOUT, {
        error: errorMessage
      });
      return;
    }
    if (!isShowMessageError) {
      return;
    }
    if (typeof handleError === 'function') {
      yield handleError(errorMessage);
    } else if (errorMessage.code === 'ERR_NETWORK') {
      _reactNative.DeviceEventEmitter.emit(_enums.EventKeys.LOST_CONNECT.toString());
    } else {
      _reactNative.DeviceEventEmitter.emit(_enums.EventKeys.SHOW_MESSAGE_ERROR.toString(), {
        error: errorMessage
      });
    }
  }
}
//# sourceMappingURL=saga.js.map