"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.loadingRepo = exports.LoadingRepository = void 0;
var _toolkit = require("@reduxjs/toolkit");
const initialState = {
  loadingActions: {},
  fetchingActions: {},
  cachingActions: {}
};
class LoadingRepository {
  static sliceName = 'loading';
  static instance(initial) {
    if (!LoadingRepository._instance) {
      LoadingRepository._instance = new LoadingRepository(initial);
    }
    return LoadingRepository._instance;
  }
  constructor(initial) {
    if (!initial) {
      this.initialize();
    } else {
      this._slices = initial();
    }
  }
  initialize() {
    this._slices = (0, _toolkit.createSlice)({
      name: LoadingRepository.sliceName,
      initialState,
      reducers: {
        showLoading: (state, action) => {
          return {
            ...state,
            loadingActions: this.addAction(state.loadingActions, action.payload)
          };
        },
        hideLoading: (state, action) => {
          return {
            ...state,
            loadingActions: this.removeAction(state.loadingActions, action.payload)
          };
        },
        onFetching: (state, action) => {
          return {
            ...state,
            fetchingActions: this.addAction(state.fetchingActions, action.payload)
          };
        },
        nonFetching: (state, action) => {
          return {
            ...state,
            fetchingActions: this.removeAction(state.fetchingActions, action.payload),
            cachingActions: this.removeAction(state.cachingActions, action.payload)
          };
        },
        onCaching: (state, action) => {
          const {
            type,
            payload
          } = action.payload;
          return {
            ...state,
            cachingActions: this.addCachingAction(state.cachingActions, type, payload)
          };
        }
      }
    });
  }
  addAction(actions, type) {
    return {
      ...actions,
      [type]: true
    };
  }
  removeAction(actions, type) {
    const action = {
      ...actions
    };
    delete action[type];
    return action;
  }
  addCachingAction(actions, type, payload) {
    return {
      ...actions,
      [type]: payload
    };
  }
  get reducer() {
    return this._slices.reducer;
  }
  get actions() {
    return this._slices.actions;
  }
  get slice() {
    return this._slices;
  }
}
exports.LoadingRepository = LoadingRepository;
const loadingRepo = LoadingRepository.instance();
exports.loadingRepo = loadingRepo;
//# sourceMappingURL=loading-slices.js.map