"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.registerStore = registerStore;
exports.rootStoreManager = void 0;
var _asyncStorage = _interopRequireDefault(require("@react-native-async-storage/async-storage"));
var _toolkit = require("@reduxjs/toolkit");
var _reduxLogger = _interopRequireDefault(require("redux-logger"));
var _reduxPersist = require("redux-persist");
var _reduxPersistTransformCompress = _interopRequireDefault(require("redux-persist-transform-compress"));
var _reduxSaga = _interopRequireDefault(require("redux-saga"));
var _effects = require("redux-saga/effects");
var _repositories = require("../repositories");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
class RootStore {
  static instance() {
    if (!RootStore._instance) {
      RootStore._instance = new RootStore();
    }
    return RootStore._instance;
  }
  whitelist = [];
  blacklist = [];
  transforms = [];
  config = {
    key: 'rootStore',
    storage: _asyncStorage.default,
    transforms: [],
    debug: true,
    whitelist: [],
    timeout: 10000
  };
  sagaMiddleware = (0, _reduxSaga.default)();
  constructor() {}
  initRootStore($storeInfo) {
    this.whitelist = $storeInfo.whiteList;
    this.blacklist = $storeInfo.blackList;
    if ($storeInfo.whiteList.length > 0) {
      var _this$transforms;
      const compressor = (0, _reduxPersistTransformCompress.default)({
        whitelist: $storeInfo.whiteList
      });
      (_this$transforms = this.transforms) === null || _this$transforms === void 0 ? void 0 : _this$transforms.push(compressor);
      this.config.whitelist = $storeInfo.whiteList;
    } else if ($storeInfo.blackList.length > 0) {
      var _this$transforms2;
      const compressor = (0, _reduxPersistTransformCompress.default)({
        blacklist: $storeInfo.blackList
      });
      (_this$transforms2 = this.transforms) === null || _this$transforms2 === void 0 ? void 0 : _this$transforms2.push(compressor);
      this.config.blacklist = $storeInfo.blackList;
    }
    this.config.transforms = this.transforms;
    const rootReducer = (0, _toolkit.combineReducers)({
      connect: _repositories.connectRepo.reducer,
      loading: _repositories.loadingRepo.reducer,
      ...$storeInfo.reducer
    });
    const persistedReducer = (0, _reduxPersist.persistReducer)(this.config, rootReducer);
    let middlewares = [];
    if (process.env.NODE_ENV === 'development') {
      middlewares.push(_reduxLogger.default);
    }
    middlewares.push(this.sagaMiddleware);
    if ($storeInfo.otherMiddlewares && $storeInfo.otherMiddlewares.length > 0) {
      middlewares.concat($storeInfo.otherMiddlewares);
    }
    this.store = (0, _toolkit.configureStore)({
      reducer: persistedReducer,
      middleware: getDefaultMiddleware => getDefaultMiddleware({
        serializableCheck: false
      }).concat(...middlewares)
    });
    function* $sagas() {
      yield (0, _effects.all)($storeInfo.sagas.map(saga => (0, _effects.fork)(saga)));
    }
    this.sagaMiddleware.run($sagas);
  }
  getStore() {
    if (!this.store) {
      console.log('The store has not been initialized!');
      return undefined;
    }
    return this.store;
  }
  getState() {
    if (!this.store) {
      console.log('The store has not been initialized!');
      return undefined;
    }
    return this.store.getState();
  }
  dispatch() {
    if (!this.store) {
      console.log('The store has not been initialized!');
      return;
    }
    return this.store.dispatch;
  }
  get getPersistStore() {
    if (!this.store) {
      throw 'The store has not been initialized!';
    }
    return (0, _reduxPersist.persistStore)(this.store, {});
  }
}
const rootStoreManager = RootStore.instance();
exports.rootStoreManager = rootStoreManager;
function registerStore($storeInfo) {
  if (!$storeInfo.reducer) {
    throw 'Error: new reducer argument must generate!';
  }
  if (rootStoreManager.getStore()) {
    console.log('The store has been initialed!');
    return;
  }
  rootStoreManager.initRootStore($storeInfo);

  // the original decorator
  function actualDecorator(_target, _property) {
    // do something with the stuff
    console.log('Genrated decorator register redux store');
  }

  // return the decorator
  return actualDecorator;
}
//# sourceMappingURL=root-store-manager.js.map