import get from 'lodash.get';
import { HttpStatusCode } from '../enums';
import { BusinessExceptionCode } from '../exceptions';
export function handleErrorMessage(error) {
  const response = get(error, 'response', null);
  if (response) {
    const status = get(response, 'status', -1);
    const message = get(response, 'message', '');
    if (status === HttpStatusCode.UNAUTHORIZED) {
      var _response$headers, _response$headers$www;
      if (response !== null && response !== void 0 && (_response$headers = response.headers) !== null && _response$headers !== void 0 && (_response$headers$www = _response$headers['www-authenticate']) !== null && _response$headers$www !== void 0 && _response$headers$www.includes('The token expired')) {
        return {
          code: HttpStatusCode.UNAUTHORIZED,
          message,
          status: BusinessExceptionCode.EXP_TOKEN
        };
      } else {
        return {
          code: HttpStatusCode.UNAUTHORIZED,
          message,
          status: BusinessExceptionCode.UNAUTHORIZED
        };
      }
    }
    if (message) {
      return {
        code: status,
        message: message,
        status: getExceptionCode(status)
      };
    }
    return {
      code: status,
      message: 'Error occurred. Please try again.',
      status: getExceptionCode(status)
    };
  } else {
    if (error && typeof error === 'object') {
      const status = get(error, 'status', -1);
      const message = get(error, 'message', '');
      const code = get(error, 'code', HttpStatusCode.BAD_REQUEST);
      if (status === HttpStatusCode.UNAUTHORIZED) {
        var _response$headers2, _response$headers2$ww;
        if (response !== null && response !== void 0 && (_response$headers2 = response.headers) !== null && _response$headers2 !== void 0 && (_response$headers2$ww = _response$headers2['www-authenticate']) !== null && _response$headers2$ww !== void 0 && _response$headers2$ww.includes('The token expired')) {
          return {
            code: status,
            message,
            status: BusinessExceptionCode.EXP_TOKEN
          };
        } else {
          return {
            code: status,
            message,
            status: BusinessExceptionCode.UNAUTHORIZED
          };
        }
      }
      if (message) {
        return {
          code: code,
          message: message,
          status: getExceptionCode(status)
        };
      }
      return {
        code: code,
        message: 'Error occurred. Please try again.',
        status: getExceptionCode(status)
      };
    }
    return {
      code: HttpStatusCode.BAD_REQUEST,
      message: 'Error occurred. Please try again.',
      status: getExceptionCode(HttpStatusCode.BAD_REQUEST)
    };
  }
}
export function getExceptionCode(statusCode) {
  switch (statusCode) {
    case HttpStatusCode.UNAUTHORIZED:
      return BusinessExceptionCode.UNAUTHORIZED;
    case HttpStatusCode.REQUEST_TIMEOUT:
      return BusinessExceptionCode.REQUEST_TIME_OUT;
    default:
      return BusinessExceptionCode.UNEXPECTED_ERROR;
  }
}
//# sourceMappingURL=handle-error-message.js.map