import { createSlice } from '@reduxjs/toolkit';
import { AppState } from 'react-native';
const initialState = {
  isConnected: true,
  appState: AppState.isAvailable ? AppState.currentState : null
};
class ConnectRepository {
  static sliceName = 'connect';
  static instance(initial) {
    if (!ConnectRepository._instance) {
      ConnectRepository._instance = new ConnectRepository(initial);
    }
    return ConnectRepository._instance;
  }
  constructor(initial) {
    if (!initial) {
      this.initialize();
    } else {
      this._slices = initial();
    }
  }
  initialize() {
    this._slices = createSlice({
      name: ConnectRepository.sliceName,
      initialState,
      reducers: {
        setConnected: (state, action) => {
          return {
            ...state,
            isConnected: action === null || action === void 0 ? void 0 : action.payload
          };
        },
        setAppState: (state, action) => {
          return {
            ...state,
            appState: action === null || action === void 0 ? void 0 : action.payload
          };
        }
      }
    });
  }
  get reducer() {
    return this._slices.reducer;
  }
  get actions() {
    return this._slices.actions;
  }
  get slice() {
    return this._slices;
  }
}
const connectRepo = ConnectRepository.instance();
export { ConnectRepository, connectRepo };
//# sourceMappingURL=connect-slices.js.map