import AsyncStorage from '@react-native-async-storage/async-storage';
import { combineReducers, configureStore } from '@reduxjs/toolkit';
import logger from 'redux-logger';
import { persistReducer, persistStore } from 'redux-persist';
import createCompressor from 'redux-persist-transform-compress';
import createSagaMiddleware from 'redux-saga';
import { all, fork } from 'redux-saga/effects';
import { connectRepo, loadingRepo } from '../repositories';
class RootStore {
  static instance() {
    if (!RootStore._instance) {
      RootStore._instance = new RootStore();
    }
    return RootStore._instance;
  }
  whitelist = [];
  blacklist = [];
  transforms = [];
  config = {
    key: 'rootStore',
    storage: AsyncStorage,
    transforms: [],
    debug: true,
    whitelist: [],
    timeout: 10000
  };
  sagaMiddleware = createSagaMiddleware();
  constructor() {}
  initRootStore($storeInfo) {
    this.whitelist = $storeInfo.whiteList;
    this.blacklist = $storeInfo.blackList;
    if ($storeInfo.whiteList.length > 0) {
      var _this$transforms;
      const compressor = createCompressor({
        whitelist: $storeInfo.whiteList
      });
      (_this$transforms = this.transforms) === null || _this$transforms === void 0 ? void 0 : _this$transforms.push(compressor);
      this.config.whitelist = $storeInfo.whiteList;
    } else if ($storeInfo.blackList.length > 0) {
      var _this$transforms2;
      const compressor = createCompressor({
        blacklist: $storeInfo.blackList
      });
      (_this$transforms2 = this.transforms) === null || _this$transforms2 === void 0 ? void 0 : _this$transforms2.push(compressor);
      this.config.blacklist = $storeInfo.blackList;
    }
    this.config.transforms = this.transforms;
    const rootReducer = combineReducers({
      connect: connectRepo.reducer,
      loading: loadingRepo.reducer,
      ...$storeInfo.reducer
    });
    const persistedReducer = persistReducer(this.config, rootReducer);
    let middlewares = [];
    if (process.env.NODE_ENV === 'development') {
      middlewares.push(logger);
    }
    middlewares.push(this.sagaMiddleware);
    if ($storeInfo.otherMiddlewares && $storeInfo.otherMiddlewares.length > 0) {
      middlewares.concat($storeInfo.otherMiddlewares);
    }
    this.store = configureStore({
      reducer: persistedReducer,
      middleware: getDefaultMiddleware => getDefaultMiddleware({
        serializableCheck: false
      }).concat(...middlewares)
    });
    function* $sagas() {
      yield all($storeInfo.sagas.map(saga => fork(saga)));
    }
    this.sagaMiddleware.run($sagas);
  }
  getStore() {
    if (!this.store) {
      console.log('The store has not been initialized!');
      return undefined;
    }
    return this.store;
  }
  getState() {
    if (!this.store) {
      console.log('The store has not been initialized!');
      return undefined;
    }
    return this.store.getState();
  }
  dispatch() {
    if (!this.store) {
      console.log('The store has not been initialized!');
      return;
    }
    return this.store.dispatch;
  }
  get getPersistStore() {
    if (!this.store) {
      throw 'The store has not been initialized!';
    }
    return persistStore(this.store, {});
  }
}
export const rootStoreManager = RootStore.instance();
export function registerStore($storeInfo) {
  if (!$storeInfo.reducer) {
    throw 'Error: new reducer argument must generate!';
  }
  if (rootStoreManager.getStore()) {
    console.log('The store has been initialed!');
    return;
  }
  rootStoreManager.initRootStore($storeInfo);

  // the original decorator
  function actualDecorator(_target, _property) {
    // do something with the stuff
    console.log('Genrated decorator register redux store');
  }

  // return the decorator
  return actualDecorator;
}
//# sourceMappingURL=root-store-manager.js.map