import { Action, AnyAction, Dispatch, Reducer, Store } from '@reduxjs/toolkit';
import { Persistor } from 'redux-persist';
import { ForkEffect } from 'redux-saga/effects';
import { IConnectState, ILoadingState } from '../repositories';
interface IStorage {
    reducer: Record<string, Reducer<any, AnyAction>>;
    sagas: (() => Generator<ForkEffect<never>, void, unknown>)[];
    whiteList: Array<string>;
    blackList: Array<string>;
    otherMiddlewares?: Array<any>;
}
interface IRootStateDefault {
    connect: IConnectState;
    loading: ILoadingState;
    [x: string]: any;
}
declare class RootStore {
    private static _instance?;
    static instance(): RootStore;
    protected whitelist: string[];
    protected blacklist: string[];
    private transforms;
    private store?;
    private config;
    private sagaMiddleware;
    constructor();
    initRootStore($storeInfo: IStorage): void;
    getStore<S extends any, A extends Action<any> = AnyAction>(): Store<S & IRootStateDefault, A> | undefined;
    getState<S extends any>(): (S & IRootStateDefault) | undefined;
    dispatch<A extends Action<any> = AnyAction>(): Dispatch<A> | undefined;
    get getPersistStore(): Persistor;
}
export declare const rootStoreManager: RootStore;
export type RootState<S = any> = S extends any ? {
    [P in keyof S]: S[P];
} & IRootStateDefault : IRootStateDefault;
export type AppDispatch = typeof rootStoreManager.dispatch;
export declare function registerStore($storeInfo: IStorage): any;
export {};
//# sourceMappingURL=root-store-manager.d.ts.map